@@ -1,4 +1,4 @@
-function summary = doctest_collect(what, directives, summary, recursive, depth, fid)
+function summary = doctest_collect(what, directives, summary, recursive, verbose, depth, fid)
 %DOCTEST_COLLECT  Find and run doctests.
 %
 %   The parameter WHAT is the name of a class, directory, function or filename:
@@ -10,8 +10,9 @@ function summary = doctest_collect(what, directives, summary, recursive, depth,
 %%
 % Copyright (c) 2010 Thomas Grenfell Smith
 % Copyright (c) 2015 Michael Walter
-% Copyright (c) 2015-2017 Colin B. Macdonald
+% Copyright (c) 2015-2019 Colin B. Macdonald
 % Copyright (c) 2015 Oliver Heimlich
+% Copyright (C) 2018 Mike Miller
 % This is Free Software, BSD-3-Clause, see doctest.m for details.
 
 
@@ -24,7 +25,12 @@ if is_octave()
   [~, ~, ext] = fileparts(what);
   if any(strcmpi(ext, {'.texinfo' '.texi' '.txi' '.tex'}))
     type = 'texinfo';
-  elseif (exist(what, 'file') && ~exist(what, 'dir')) || exist(what, 'builtin');
+  elseif (strcmp (ext, '.oct') && exist (what) == 3)  % .oct explicitly
+    type = 'octfile';
+  elseif (exist (what) == 3)  % .oct/.mex
+    [~, what, ~] = fileparts (what);  % strip extension if present
+    type = 'function';                % then access like any function
+  elseif (exist(what, 'file') && ~exist(what, 'dir')) || exist(what, 'builtin')
     if (exist(['@' what], 'dir'))
       % special case, e.g., @logical is class, logical is builtin
       type = 'class';
@@ -88,7 +94,9 @@ if (strcmp(type, 'dir'))
     else
       slashchar = filesep();
     end
-    fprintf(fid, '%s%s%s\n', spaces, what, slashchar);
+    if (verbose)
+      fprintf(fid, '%s%s%s\n', spaces, what, slashchar);
+    end
   end
   oldcwd = chdir(what);
   files = dir('.');
@@ -109,12 +117,13 @@ if (strcmp(type, 'dir'))
       end
     else
       [~, ~, ext] = fileparts(f);
-      if (~ any(strcmpi(ext, {'.m' '.texinfo' '.texi' '.txi' '.tex'})))
+      if (~ any(strcmpi(ext, ...
+                {'.m' '.texinfo' '.texi' '.txi' '.tex' '.oct' '.mex'})))
         %fprintf(fid, 'Debug: ignoring file "%s"\n', f)
         continue
       end
     end
-    summary = doctest_collect(f, directives, summary, recursive, depth + 1, fid);
+    summary = doctest_collect(f, directives, summary, recursive, verbose, depth + 1, fid);
   end
   chdir(oldcwd);
   return
@@ -135,6 +144,8 @@ if strcmp(type, 'function')
   targets = [target];
 elseif strcmp(type, 'class')
   targets = collect_targets_class(what, depth);
+elseif strcmp (type, 'octfile')
+  targets = collect_targets_octfile (what, depth);
 elseif strcmp(type, 'texinfo')
   target = struct();
   target.name = what;
@@ -165,13 +176,17 @@ for i=1:numel(targets)
   target = targets(i);
   spaces = repmat(' ', 1, 2*target.depth);
   dots = repmat('.', 1, 55 - numel(target.name) - 2*target.depth);
-  fprintf(fid, '%s%s %s ', spaces, target.name, dots);
+  if (verbose)
+    fprintf(fid, '%s%s %s ', spaces, target.name, dots);
+  end
 
   % extraction error?
   if target.error
     summary.num_targets_with_extraction_errors = summary.num_targets_with_extraction_errors + 1;
-    fprintf(fid, [color_err  'EXTRACTION ERROR' reset '\n\n']);
-    fprintf(fid, '    %s\n\n', target.error);
+    if (verbose)
+      fprintf(fid, [color_err  'EXTRACTION ERROR' reset '\n\n']);
+      fprintf(fid, '    %s\n\n', target.error);
+    end
     continue;
   end
 
@@ -197,22 +212,24 @@ for i=1:numel(targets)
     summary.num_targets_without_tests = summary.num_targets_without_tests + 1;
   end
 
-  % pretty print outcome
-  if num_tests == 0
-    fprintf(fid, 'NO TESTS\n');
-  elseif num_tests_passed == num_tests
-    fprintf(fid, [color_ok 'PASS %4d/%-4d' reset '\n'], num_tests_passed, num_tests);
-  else
-    fprintf(fid, [color_err 'FAIL %4d/%-4d' reset '\n\n'], num_tests - num_tests_passed, num_tests);
-    for j = 1:num_tests
-      if ~results(j).passed
-        fprintf(fid, '   >> %s\n\n', results(j).source);
-        fprintf(fid, [ '      expected: ' '%s' '\n' ], results(j).want);
-        fprintf(fid, [ '      got     : ' color_err '%s' reset '\n' ], results(j).got);
-        if results(j).xfail
-          fprintf(fid, '      expected failure, but test succeeded!');
+  if (verbose)
+    % pretty print outcome
+    if num_tests == 0
+      fprintf(fid, 'NO TESTS\n');
+    elseif num_tests_passed == num_tests
+      fprintf(fid, [color_ok 'PASS %4d/%-4d' reset '\n'], num_tests_passed, num_tests);
+    else
+      fprintf(fid, [color_err 'FAIL %4d/%-4d' reset '\n\n'], num_tests - num_tests_passed, num_tests);
+      for j = 1:num_tests
+        if ~results(j).passed
+          fprintf(fid, '   >> %s\n\n', results(j).source);
+          fprintf(fid, [ '      expected: ' '%s' '\n' ], results(j).want);
+          fprintf(fid, [ '      got     : ' color_err '%s' reset '\n' ], results(j).got);
+          if results(j).xfail
+            fprintf(fid, '      expected failure, but test succeeded!');
+          end
+          fprintf(fid, '\n');
         end
-        fprintf(fid, '\n');
       end
     end
   end
@@ -283,6 +300,35 @@ function targets = collect_targets_class(what, depth)
 end
 
 
+function targets = collect_targets_octfile (file, depth)
+  % first target is the name of the octfile (w/o extension)
+  [~, basename, ext] = fileparts (file);
+  assert (strcmp (ext, '.oct'))
+  target = collect_targets_function (basename);
+  target.name = file;
+  target.depth = depth;
+  targets = [target];
+
+  % octfile may have many fcns in it: find them using the autoload map
+  autoloadmap = autoload ();
+  len = numel (file);
+  % matches both "/foo/bar.oct" and "/baz/bar.oct"; uncommon in practice
+  pmatch = @(e) (numel (e.file) >= len) && strcmp (e.file(end-len+1:end), file);
+  idx = find (arrayfun (pmatch, autoloadmap));
+
+  if (~ isempty (idx))
+    % indicate that octfile has other fcns, and indent those targets
+    targets(1).name = [targets(1).name ':'];
+    for i = 1:numel (idx)
+      f = autoloadmap(idx(i)).function;
+      target = collect_targets_function (f);
+      target.depth = depth + 1;
+      targets = [targets; target];
+    end
+  end
+end
+
+
 function [docstring, error] = extract_docstring(name)
   if is_octave()
     [docstring, format] = get_help_text(name);
@@ -341,6 +387,7 @@ function [docstring, error] = parse_texinfo(str)
 
   % These should work, but I keep hitting ARM-specific when $1 is empty:
   % https://savannah.gnu.org/bugs/index.php?52810
+  % TODO: fixed in 4.2.2, so can revert this once we drop 4.2.1 support
   %str = regexprep (str, ...
   %                 '^([ \t]*)(\@example)(.*)$', ...
   %                 [ '$1$2$3\n', ... % retain original line
